# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Can use either module directly:
- API: Django REST
- Editor model: (load, save, historify, execute...) doc2 model, connector models, user models
- Editor executor: engine (zero Django/models but dict inputs: interpreter, username). Editor API with execute and /autocomp
- SQL client: sqlalchemy_client (native session_id, job_id), Flink? --> make a dialect instead. SQL based only. Live queries?
"""

from compose.editor.query.sqlalchemy_api import SqlAlchemyInterface

SESSIONS = {}
HANDLES = {}


class Executor:
    """
    Compose specific: highest/simplest possible, Editor API, combo Connectors/Dialects/Sessions data under hood
    Pure exec is in Client (sqlalchemy)
    Hue UUID wrapper around native handle, manage caches, dialects
    Session reuse / Cache py or persistence
    Goal is to support connectors, return uuid, offer reuse of session or not. Flink INSERT job, CREATE MODEL --> Hue id (Multi concurrent queries)
    Easy support via Task, scheduled Task, importer create table "jobs", Result Explorer tasks ...
    """

    def __init__(self, username, interpreter):
        self.username = username
        self.interpreter = interpreter
        self.connector = SqlAlchemyInterface(
            username, interpreter
        )  # Only SqlAlchemy as interface --> client

    # For under the cover operations like install examples
    def query(self, statement):
        query = {
            "statement": statement,
            "database": None,
            "dialect": self.interpreter["dialect"],
        }
        return self.connector.query(query)

    # Query Object? simple statement?
    # execute --> query
    def execute(self, statement):
        # sessions id or None or auto?
        # hue uuid/id(if historify) + native handle
        # async if TS, check poll/stream too

        # interpreter.execute needs the sessions, but we don't want to persist them
        # pre_execute_sessions = notebook['sessions']
        # notebook['sessions'] = sessions
        # session = self._get_session(notebook, snippet)
        query = {
            "statement": statement,
            "database": None,
            "dialect": self.interpreter["dialect"],
        }
        handle = self.connector.execute(query)
        # notebook['sessions'] = pre_execute_sessions

        # TODO: here could integrate with Hue Documents when query history is set
        # handle["history_id"] = 1396
        # handle["history_uuid"] = "d1799c55-8518-4e68-9545-26043954269f"

        return handle

    def check_status(self, query_id):
        data = self.connector.check_status(query_id)

        return {"status": data["status"]}

    def fetch_result(self, query_id, rows=100, start_over=False):
        return self.connector.fetch_result(query_id, rows=rows, start_over=start_over)

    def autocomplete(self, database=None, table=None, column=None, nested=None):
        data = self.connector.autocomplete(
            database=database, table=table, column=column, nested=nested
        )

        return data


# class ExecutorTracer():

#     def pre_execute(self, *args, **kwargs):
#         with opentracing.tracer.start_span("notebook-execute") as span:
#             span.set_tag("user-id", self.username)

#     def post_execute(self):
#         span.set_tag("query-id", response.get("handle", {}).get("guid"))


# class Executor
#  query
#  execute / fetch_status / fetch_results / fetch_logs
#  autocomplete
#  explain (smart)
#  schedule
# class SessionExecutor
#      ... Tracer
# class HistorifyExecutor


def query_error_handler(func):
    def decorator(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except OperationalError as e:
            message = str(e)
            if "1045" in message:  # 'Access denied' # MySQL
                raise AuthenticationRequired(message=message)
            else:
                raise e
        except AuthenticationRequired:
            raise
        except QueryExpired:
            raise
        except Exception as e:
            message = force_unicode(e)
            if (
                "Invalid query handle" in message
                or "Invalid OperationHandle" in message
            ):
                raise QueryExpired(e)
            else:
                LOG.exception("Query Error")
                raise QueryError(message)

    return decorator



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rest_framework.decorators import api_view
from rest_framework.response import Response


@api_view(["POST"])
def get_types(request):
    return Response({"connectors": [], "categories": []})


@api_view(["POST"])
def get_instances(request):
    return Response({"connectors": []})


@api_view(["POST"])
def get_config(request):
    return Response(
        {
            "app_config": {
                "editor": {
                    "name": "editor",
                    "displayName": "Editor",
                    "buttonName": "Query",
                    "interpreters": [
                        {
                            "name": "MySQL",
                            "type": "1",
                            "id": "1",
                            "displayName": "MySQL",
                            "buttonName": "Query",
                            "tooltip": "MySQL Query",
                            "optimizer": "api",
                            "page": "/editor/?type=1",
                            "is_sql": True,
                            "is_batchable": True,
                            "dialect": "mysql",
                            "dialect_properties": {
                                "is_sql": True,
                                "sql_identifier_quote": "`",
                                "sql_identifier_comment_single": "--",
                                "has_catalog": True,
                                "has_database": True,
                                "has_table": True,
                                "has_live_queries": False,
                                "has_optimizer_risks": True,
                                "has_optimizer_values": True,
                                "has_auto_limit": False,
                                "has_reference_language": False,
                                "has_reference_functions": False,
                                "has_use_statement": False,
                                "trim_statement_semicolon": False,
                            },
                        }
                    ],
                    "default_limit": 5000,
                    "interpreter_names": ["1"],
                    "page": "/editor/?type=1",
                    "default_sql_interpreter": "1",
                },
                "catalogs": [],
                "browser": {},
                "home": {
                    "name": "home",
                    "displayName": "Home",
                    "buttonName": "Saved Queries",
                    "interpreters": [],
                    "page": "/home",
                },
            },
            "main_button_action": {},
            "button_actions": [],
            "default_sql_interpreter": "1",
            "cluster_type": "direct",
            "has_computes": False,
            "hue_config": {"enable_sharing": True, "is_admin": True},
            "clusters": [
                {
                    "id": "default",
                    "name": "demo.gethue.com",
                    "type": "direct",
                    "credentials": {},
                }
            ],
            "documents": {"types": ["query-mysql", "directory"]},
            "status": 0,
        }
    )


@api_view(["GET"])
def get_namespaces(request, interface):
    return Response(
        {
            "1": [
                {
                    "id": "default",
                    "name": "default",
                    "status": "CREATED",
                    "computes": [
                        {
                            "id": "default",
                            "name": "default",
                            "type": "direct",
                            "credentials": {},
                        }
                    ],
                }
            ],
            "status": 0,
        }
    )



#!/usr/bin/env python
# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging

from django.http import HttpResponseBadRequest, JsonResponse
from django.utils.functional import wraps
from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import OpenApiExample, extend_schema
from rest_framework.decorators import api_view

from .query.engines import Executor
from .query.exceptions import QueryError, QueryExpired

LOG = logging.getLogger(__name__)


def api_error_handler(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        response = {}

        try:
            return f(*args, **kwargs)
        except QueryExpired:
            response["query_status"] = {"status": "expired"}
            response["status"] = 0
        except QueryError as e:
            LOG.exception("Error running %s" % f.__name__)
            response["status"] = 1
            response["message"] = e.message
        finally:
            if response:
                return JsonResponse(response)

    return wrapper


@extend_schema(
    description="Sync SQL statement execution",
    request={"application/json": OpenApiTypes.OBJECT},
    responses=OpenApiTypes.STR,
)
@api_view(["POST"])
def query(request, dialect=None):
    statement = request.data.get("statement")

    if not statement:
        return HttpResponseBadRequest()

    interpreter = _get_interpreter(request.data)

    data = Executor(username=request.user, interpreter=interpreter).query(
        statement=statement
    )

    return JsonResponse(data)


@api_view(["POST"])
def create_session(request):
    return JsonResponse(
        {"status": 0, "session": {"type": "1", "id": None, "properties": []}}
    )


@extend_schema(
    description="Async SQL statement execution",
    request=OpenApiTypes.STR,
    responses=OpenApiTypes.STR,
)
@api_view(["POST"])
@api_error_handler
def execute(request, dialect=None):
    statement = (
        request.data.get("statement")
        or json.loads(request.data.get("executable"))["statement"]
    )
    interpreter = _get_interpreter(request.data)

    if not statement:
        return HttpResponseBadRequest()

    handle = Executor(username=request.user, interpreter=interpreter).execute(
        statement=statement
    )

    return JsonResponse(
        {
            "status": 0,
            "handle": handle,
            "history_id": 1396,  # To push down to Engine but needed in current js
            "history_uuid": handle["guid"],  # Same
        }
    )


@api_view(["POST"])
@api_error_handler
def check_status(request):
    query_id = request.data.get("query_id")
    if not query_id:
        query_id = request.data.get("operationId")

    interpreter = _get_interpreter(query_id)

    data = Executor(username=request.user, interpreter=interpreter).check_status(
        query_id=query_id
    )

    # Note: {"status": "available", "has_result_set": true}
    return JsonResponse({"status": 0, "query_status": data})


@api_view(["POST"])
@api_error_handler
def fetch_result_data(request):
    query_id = request.data.get("operationId")
    rows = json.loads(request.data.get("rows", "100"))
    start_over = json.loads(request.data.get("startOver", "false"))

    interpreter = _get_interpreter(query_id)

    response = Executor(username=request.user, interpreter=interpreter).fetch_result(
        query_id=query_id, rows=rows, start_over=start_over
    )

    return JsonResponse(
        {
            "result": response,
            "status": 0,
        }
    )


@api_view(["POST"])
@api_error_handler
def get_logs(request):
    return JsonResponse(
        {"status": 0, "logs": "", "progress": 50, "jobs": [], "isFullLogs": True}
    )


@api_view(["POST"])
@api_error_handler
def cancel(request):
    return JsonResponse({})


@api_view(["POST"])
@api_error_handler
def close(request):
    return JsonResponse({})


@api_view(["POST"])
@api_error_handler
def close_session(request):
    return JsonResponse({})


@api_view(["POST"])
@api_error_handler
def autocomplete(request, database=None, table=None, column=None, nested=None):
    print(request.data)
    print(request.POST)

    interpreter = _get_interpreter(request.data)

    data = Executor(username=request.user, interpreter=interpreter).autocomplete(
        database=database, table=table, column=column, nested=nested
    )

    return JsonResponse(data)


def _get_interpreter(query_id):
    # TODO
    # From query_id get connector_id and so get interpreter
    # connector_id = ...

    interpreter = {
        "options": {
            "url": "sqlite:///db-demo.sqlite3"  # Dev server only (single threading)
            # "url": "mysql://hue:hue@localhost:3306/hue"
        },
        "name": "mysql",
        "dialect_properties": {},
        "dialect": "mysql",
    }

    return interpreter


# API specs
# Operation: https://swagger.io/specification/#operation-object
# Some possibilities, obviously going more manual if we don't have models. Can be case per case depending on API popularity.
# - Manual text docs like on current API docs
# - Reuse/Create a DRF serializer
# - Manual request to text and example object
# - Manual request body (and response)
@extend_schema(
    description="Minimal API for submitting an SQL statement synchronously",
    request={"application/json": OpenApiTypes.OBJECT},
    responses=OpenApiTypes.STR,
    examples=[
        OpenApiExample(
            name="SELECT 1, 2, 3",
            value={"statement": "SELECT 1, 2, 3"},
        )
    ],
    # Full override, all manual
    # https://github.com/tfranzel/drf-spectacular/issues/279
    # https://github.com/tfranzel/drf-spectacular/blob/6f12e8d9310ca2aaa833a1167d0d5f7795e2d635/tests/test_extend_schema.py#L160-L186
    # Note: seems to lose Parameters when set?
    operation={
        "operationId": "manual_endpoint",
        "tags": ["editor"],
        # https://swagger.io/specification/#request-body-object
        "requestBody": {
            "content": {
                "application/json": {
                    "schema": {
                        "type": "array",
                        "items": {"type": "string"},
                    },
                    "examples": {
                        "1, 2, 3": {
                            "summary": "List of numbers",
                            "value": ["1", "2", "3"],
                        },
                    },
                    # "schema": {
                    #     "type": "object",
                    #     "properties": {"statement": {"type": "string"}},
                    #     "example": {"statement": "SELECT 1, 2, 3"}
                    # },
                }
            },
        },
    },
)
@api_view(["POST"])
def hello(request, message=None):
    print(request.data)
    print(request.POST)

    return JsonResponse({"data": request.data, "message": message})



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from pathlib import Path

import requests
import typer
import yaml

app = typer.Typer()

APP_NAME = "compose"

"""
Wrapper around https://docs.gethue.com/developer/api/rest/
"""


@app.callback()
def callback():
    """
    Query your Data Easily
    """


@app.command()
def auth(
    api_url: str = typer.Option("https://demo.gethue.com", prompt=True),
    username: str = typer.Option("demo", prompt=True),
    password: str = typer.Option("demo", prompt=True, hide_input=True),
):
    """
    Authenticate with the API server
    """
    session = requests.Session()

    data = {
        "username": username,
        "password": password,
    }

    response = session.post("%s/api/token/auth" % api_url, data=data)
    print(
        "Auth: %s %s"
        % ("success" if response.status_code == 200 else "error", response.status_code)
    )

    token = json.loads(response.content)["access"]
    print("Token: %s" % token)

    save_api_creds(api_url, username, token)


@app.command()
def query():
    """
    Execute SQL statements: TBD

    """
    typer.echo("Execute SQL")

    creds = get_config()["auth"]

    response = requests.post(
        "%(url)s/api/query/autocomplete" % creds,
        headers={
            "Authorization": "Bearer %(token)s" % creds,
            "Content-Type": "application/x-www-form-urlencoded",
        },
        data={"snippet": json.dumps({"type": "mysql"})},
    )
    print(response.status_code)
    print(response.text)


@app.command()
def storage():
    """
    Manipulate data files: TBD
    """
    typer.echo("List, upload, download data files")


@app.command()
def importer():
    """
    Create table wizard from data: TBD
    """
    typer.echo("Create table from a file")


@app.command()
def connectors():
    """
    Manipulate connections to databases and storages: TBD
    """
    typer.echo("List, edit connector instances")


@app.command()
def catalog():
    """
    Manipulate Catalog metadata: TBD
    """
    typer.echo("Search, list and update table description")


def save_api_creds(api_url: str, username: str, token: str):
    app_dir = typer.get_app_dir(APP_NAME)
    config_dir = Path(app_dir)
    config_dir.mkdir(exist_ok=True)
    config_path: Path = config_dir / "config.yml"

    yaml.dump(
        {"auth": {"url": api_url, "username": username, "token": token}},
        open(config_path, "w"),
    )


def get_config():
    app_dir = typer.get_app_dir(APP_NAME)
    config_path: Path = Path(app_dir) / "config.yml"

    if not config_path.is_file():
        typer.echo(
            "Config file doesn't exist yet. Please run again the `auth` command."
        )

    config = yaml.safe_load(open(config_path))

    return config


if __name__ == "__main__":
    app()



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path

from . import api

app_name = "connector"


urlpatterns = [
    re_path(r"^api2/get_config/?$", api.get_config, name="get_config"),
    re_path(  # To not support anymore going forward, unused
        r"^api2/context/namespaces/(?P<id>\d+)/?$",
        api.get_namespaces,
        name="get_namespaces",
    ),
]



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Django settings for project.

Generated by 'django-admin startproject' using Django 3.1.6.

For more information on this file, see
https://docs.djangoproject.com/en/3.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.1/ref/settings/
"""

import datetime
from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
RELATIVE_DIR = Path(__file__).resolve().parent
BASE_DIR = RELATIVE_DIR.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "zj^!-wlzr8y^7%9$+yyxnxeh%^8p9(vf-vrvsi^1f1ylrk05sv"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = [
    "0.0.0.0",
    "127.0.0.1",
    "localhost",
    "api.gethue.com",
]

ATOMIC_REQUESTS = True


INSTALLED_APPS = [
    "compose.editor.apps.EditorConfig",
    "compose.connectors.apps.ConnectorsConfig",
    "corsheaders",
    # "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "whitenoise.runserver_nostatic",  # Before staticfiles
    "django.contrib.staticfiles",
    "drf_spectacular",
    "rest_framework",
    "rest_framework.authtoken",
]

MIDDLEWARE = [
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    # 'django.middleware.csrf.CsrfViewMiddleware',
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "compose.conf.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "conf.wsgi.application"

STATIC_ROOT = "static"

# Forever-cacheable files and compression support
STATICFILES_STORAGE = "whitenoise.storage.CompressedManifestStaticFilesStorage"
WHITENOISE_INDEX_FILE = True

TEST_RUNNER = "compose.conf.runner.PytestTestRunner"


# Database
# https://docs.djangoproject.com/en/3.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": str(BASE_DIR / "db.sqlite3"),
    }
}

CORS_ALLOW_ALL_ORIGINS = True
CORS_URLS_REGEX = r"^/api/.*$"
CORS_ALLOW_CREDENTIALS = True

# CORS_ALLOWED_ORIGINS = [
#     "https://example.com",
#     "https://sub.example.com",
#     "http://localhost:8080",
#     "http://127.0.0.1:9000"
# ]
# CORS_ALLOWED_ORIGIN_REGEXES = [
#     r"^https://\w+\.example\.com$",
# ]

REST_FRAMEWORK = {
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.IsAuthenticated",
        # "rest_framework.permissions.AllowAny"
    ],
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework_simplejwt.authentication.JWTAuthentication",
        "rest_framework.authentication.SessionAuthentication",
    ),
    "DEFAULT_RENDERER_CLASSES": [
        "rest_framework.renderers.JSONRenderer",
        "rest_framework.renderers.BrowsableAPIRenderer",
    ],
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "DEFAULT_PARSER_CLASSES": [
        "rest_framework.parsers.JSONParser",
        "rest_framework.parsers.FormParser",
        "rest_framework.parsers.MultiPartParser",
    ],
}

JWT_AUTH = {
    "JWT_VERIFY": True,
    "JWT_VERIFY_EXPIRATION": True,
    "JWT_LEEWAY": 0,
    "JWT_EXPIRATION_DELTA": datetime.timedelta(seconds=86400),
    "JWT_ALLOW_REFRESH": True,
    "JWT_REFRESH_EXPIRATION_DELTA": datetime.timedelta(days=7),
    "JWT_AUTH_HEADER_PREFIX": "Bearer",
}

# Password validation
# https://docs.djangoproject.com/en/3.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.1/howto/static-files/

STATIC_URL = "/static/"
MEDIA_URL = "/media/"

# Parameterization
try:
    from compose.conf.local_settings import *  # noqa
except ImportError:
    from shutil import copyfile

    copyfile(
        RELATIVE_DIR / "local_settings_template.py", RELATIVE_DIR / "local_settings.py"
    )



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path

from . import api

app_name = "editor"

urlpatterns = [
    re_path(r"^create_session/?$", api.create_session, name="api_create_session"),
    re_path(r"^close_session/?$", api.close_session, name="api_close_session"),
    re_path(r"^query/(?P<dialect>.+)/?$", api.query, name="query"),
    re_path(r"^execute/(?P<dialect>.+)/?$", api.execute, name="execute"),
    re_path(r"^check_status/?$", api.check_status, name="check_status"),
    re_path(r"^fetch_result_data/?$", api.fetch_result_data, name="fetch_result_data"),
    re_path(r"^get_logs/?$", api.get_logs, name="get_logs"),
    re_path(r"^cancel_statement/?$", api.cancel, name="cancel_statement"),
    re_path(r"^close_statement/?$", api.close, name="close_statement"),
    re_path(r"^autocomplete/?$", api.autocomplete, name="autocomplete_databases"),
    re_path(
        r"^autocomplete/(?P<database>[^/?]*)/?$",
        api.autocomplete,
        name="autocomplete_tables",
    ),
    re_path(
        r"^autocomplete/(?P<database>[^/?]*)/(?P<table>[\w_\-]+)/?$",
        api.autocomplete,
        name="autocomplete_columns",
    ),
    re_path(
        r"^autocomplete/(?P<database>[^/?]*)/(?P<table>[\w_\-]+)/(?P<column>\w+)/?$",
        api.autocomplete,
        name="autocomplete_column",
    ),
    re_path(
        r"^autocomplete/(?P<database>[^/?]*)/(?P<table>[\w_\-]+)/(?P<column>\w+)/(?P<nested>.+)/?$",
        api.autocomplete,
        name="autocomplete_nested",
    ),
    re_path(r"^hello/?$", api.hello, name="hello"),
]



#!/usr/bin/env python
# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import json
import logging
import re
import textwrap
import uuid
from string import Template

from sqlalchemy import MetaData, Table, create_engine, inspect
from sqlalchemy.exc import (
    CompileError,
    NoSuchTableError,
    OperationalError,
    ProgrammingError,
    UnsupportedCompilationError,
)

from compose.editor.query.exceptions import (
    AuthenticationRequired,
    QueryError,
    QueryExpired,
)

ENGINES = {}  # Sessions
CONNECTIONS = {}  # Query Handles
ENGINE_KEY = "%(username)s-%(connector_name)s"
URL_PATTERN = "(?P<driver_name>.+?://)(?P<host>[^:/ ]+):(?P<port>[0-9]*).*"

LOG = logging.getLogger(__name__)


def query_error_handler(func):
    def decorator(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except (OperationalError, ProgrammingError) as e:
            raise QueryError(ex=e)

    return decorator


# Api vs Client
# Handle the DB client. Also reuse qhandles, session caches?
class SqlAlchemyInterface:
    def __init__(self, username, interpreter):
        # super(SqlAlchemyApi, self).__init__(user=user, interpreter=interpreter)
        self.username = username
        self.interpreter = interpreter

        self.options = interpreter["options"]

        if interpreter.get("dialect_properties"):
            self.backticks = interpreter["dialect_properties"]["sql_identifier_quote"]
        else:
            self.backticks = (
                '"'
                if re.match(
                    "^(postgresql://|awsathena|elasticsearch|phoenix)",
                    self.options.get("url", ""),
                )
                else "`"
            )

    def _get_engine_key(self):  # --> to Executor?
        return ENGINE_KEY % {
            "username": self.username,
            "connector_name": self.interpreter["name"],
        }

    def _get_engine(self):
        engine_key = self._get_engine_key()

        if engine_key not in ENGINES:
            ENGINES[engine_key] = self._create_engine()

        return ENGINES[engine_key]

    def _create_engine(self):
        if (
            "${" in self.options["url"]
        ):  # URL parameters substitution, should be in Engine as custom to Hue globally
            vars = {"USER": self.username}

            if "${PASSWORD}" in self.options["url"]:
                auth_provided = False
                if "session" in self.options:
                    for _prop in self.options["session"]["properties"]:
                        if _prop["name"] == "user":
                            vars["USER"] = _prop["value"]
                            auth_provided = True
                        if _prop["name"] == "password":
                            vars["PASSWORD"] = _prop["value"]
                            auth_provided = True

                if not auth_provided:
                    raise AuthenticationRequired(
                        message="Missing username and/or password"
                    )

            raw_url = Template(self.options["url"])
            url = raw_url.safe_substitute(**vars)
        else:
            url = self.options["url"]

        # --> to move to py Hooks in connector types
        if url.startswith("awsathena+rest://"):
            url = url.replace(url[17:37], urllib_quote_plus(url[17:37]))
            url = url.replace(url[38:50], urllib_quote_plus(url[38:50]))
            s3_staging_dir = url.rsplit("s3_staging_dir=", 1)[1]
            url = url.replace(s3_staging_dir, urllib_quote_plus(s3_staging_dir))

        if self.options.get("has_impersonation"):
            m = re.search(URL_PATTERN, url)
            driver_name = m.group("driver_name")

            if not driver_name:
                raise QueryError(
                    "Driver name of %(url)s could not be found and impersonation is turned on"
                    % {"url": url}
                )

            url = url.replace(
                driver_name,
                "%(driver_name)s%(username)s@"
                % {"driver_name": driver_name, "username": self.username},
            )

        if self.options.get("credentials_json"):
            self.options["credentials_info"] = json.loads(
                self.options.pop("credentials_json")
            )

        # Enables various SqlAlchemy args to be passed along for both Hive & Presto connectors
        # Refer to SqlAlchemy pyhive for more details
        if self.options.get("connect_args"):
            self.options["connect_args"] = json.loads(self.options.pop("connect_args"))

        options = self.options.copy()
        options.pop("session", None)
        options.pop("url", None)
        options.pop("has_ssh", None)
        options.pop("has_impersonation", None)
        options.pop("ssh_server_host", None)

        options["pool_pre_ping"] = True

        return create_engine(url, **options)

    def _get_session(self, notebook, snippet):
        for session in notebook["sessions"]:
            if session["type"] == snippet["type"]:
                return session

        return None

    def _create_connection(self, engine):
        connection = None
        try:
            connection = engine.connect()
        except Exception as e:
            engine_key = self._get_engine_key()
            ENGINES.pop(engine_key, None)

            raise AuthenticationRequired(
                message="Could not establish connection to datasource: %s" % e
            )

        return connection

    def query(self, query):
        return self.execute(query, is_async=False)

    @query_error_handler
    def execute(self, query, is_async=True):
        guid = uuid.uuid4().hex

        # session = self._get_session(notebook, snippet)
        # if session is not None:
        #     self.options["session"] = session

        engine = self._get_engine()
        connection = self._create_connection(engine)
        statement = query["statement"]

        if self.interpreter["dialect_properties"].get("trim_statement_semicolon", True):
            statement = statement.strip().rstrip(";")

        if self.interpreter["dialect_properties"].get(
            "has_use_statement"
        ) and query.get("database"):
            connection.execute(
                "USE %(sql_identifier_quote)s%(database)s%(sql_identifier_quote)s"
                % {
                    "sql_identifier_quote": self.interpreter["dialect_properties"][
                        "sql_identifier_quote"
                    ],
                    "database": snippet["database"],
                }
            )

        result = connection.execute(statement)

        # cache == sa_query_handle
        cache = {
            "connection": connection,  # Session
            "result": result,  # Handle
            "meta": [
                {
                    "name": col[0]
                    if type(col) is tuple or type(col) is dict
                    else col.name
                    if hasattr(col, "name")
                    else col,
                    "type": "STRING_TYPE",
                    "comment": "",
                }
                for col in result.cursor.description
            ]
            if result.cursor
            else [],
            "has_result_set": result.cursor != None,
        }
        CONNECTIONS[guid] = cache

        return {
            "sync": not is_async,
            "has_result_set": cache["has_result_set"],
            "modified_row_count": 0,
            "guid": guid,  # Rename to id
            "result": {
                "has_more": result.cursor != None,
                "data": []
                if is_async
                else [[col for col in row] for row in result.fetchmany(10)],
                "meta": cache["meta"],
                "type": "table",
            },
        }

    @query_error_handler
    def check_status(self, query_id):
        handle = CONNECTIONS.get(query_id)

        response = {"status": "canceled"}

        if handle:
            cursor = handle["result"].cursor
            if self.options["url"].startswith("presto://") and cursor and cursor.poll():
                response["status"] = "running"
            elif handle["has_result_set"]:
                response["status"] = "available"
            else:
                response["status"] = "success"
        else:
            raise QueryExpired()

        return response

    @query_error_handler
    def fetch_result(self, query_id, rows=100, start_over=False):
        handle = CONNECTIONS.get(query_id)

        if handle:
            data = [[col for col in row] for row in handle["result"].fetchmany(rows)]
            meta = handle["meta"]
            self._assign_types(data, meta)
        else:
            raise QueryExpired()

        return {
            "has_more": len(data) >= rows,
            "data": data,
            "meta": meta,
            "type": "table",
        }

    @query_error_handler
    def autocomplete(
        self,
        database=None,
        table=None,
        column=None,
        nested=None,
        operation=None,
    ):
        if self.interpreter["dialect"] == "phoenix":
            if database:
                database = database.upper()
            if table:
                table = table.upper()
        engine = self._get_engine()
        inspector = inspect(engine)

        assist = Assist(inspector, engine, backticks=self.backticks)
        response = {"status": -1}

        if operation == "functions":
            response["functions"] = []
        elif operation == "function":
            response["function"] = {}
        elif database is None:
            response["databases"] = [db or "" for db in assist.get_databases()]
        elif table is None and operation == "models":
            response["models"] = [t for t in assist.get_models(database)]
        elif table is None:
            tables_meta = []
            for t in assist.get_table_names(database):
                t = self._fix_bigquery_db_prefixes(t)
                tables_meta.append({"name": t, "type": "Table", "comment": ""})
            for t in assist.get_view_names(database):
                t = self._fix_bigquery_db_prefixes(t)
                tables_meta.append({"name": t, "type": "View", "comment": ""})
            response["tables_meta"] = tables_meta
        elif column is None:
            columns = assist.get_columns(database, table)

            response["columns"] = [col["name"] for col in columns]
            response["extended_columns"] = [
                {
                    "autoincrement": col.get("autoincrement"),
                    "comment": col.get("comment"),
                    "default": col.get("default"),
                    "name": col.get("name"),
                    "nullable": col.get("nullable"),
                    "type": self._get_column_type_name(col),
                }
                for col in columns
            ]

            if (
                not self.options["url"].startswith("phoenix://")
                and operation != "model"
            ):
                response.update(assist.get_keys(database, table))
        else:
            columns = assist.get_columns(database, table)
            response["name"] = next(
                (col["name"] for col in columns if column == col["name"]), ""
            )
            response["type"] = next(
                (str(col["type"]) for col in columns if column == col["name"]), ""
            )

        response["status"] = 0
        return response

    def _assign_types(self, results, meta):
        result = results and results[0]
        if result:
            for index, col in enumerate(result):
                if isinstance(col, int):
                    meta[index]["type"] = "INT_TYPE"
                elif isinstance(col, float):
                    meta[index]["type"] = "FLOAT_TYPE"
                elif isinstance(col, bool):
                    meta[index]["type"] = "BOOLEAN_TYPE"
                elif isinstance(col, datetime.date):
                    meta[index]["type"] = "TIMESTAMP_TYPE"
                else:
                    meta[index]["type"] = "STRING_TYPE"

    def _get_column_type_name(self, col):
        try:
            name = str(col.get("type"))
        except (UnsupportedCompilationError, CompileError):
            name = col.get("type").__visit_name__.lower()

        return name

    def _fix_bigquery_db_prefixes(self, identifier):
        if type(identifier) == dict and identifier.get("type") == "model":
            identifier["type"] = "Model"

        if self.options["url"].startswith("bigquery://"):
            identifier["name"] = identifier["name"].rsplit(".", 1)[-1]

        return identifier


class Assist(object):
    def __init__(self, db, engine, backticks, api=None):
        self.db = db
        self.engine = engine
        self.backticks = backticks
        self.api = api

    def get_databases(self):
        return self.db.get_schema_names()

    def get_table_names(self, database, table_names=[]):
        return self.db.get_table_names(database)

    def get_view_names(self, database, view_names=[]):
        return self.db.get_view_names(database)

    def get_tables(self, database, table_names=[]):
        return self.get_table_names(database) + self.get_view_names(database)

    def get_models(self, database):
        return [
            t for t in self.db.get_table_names(database) if isinstance(t, dict)
        ]  # Currently only supported by pybigquery

    def get_columns(self, database, table):
        try:
            return self.db.get_columns(table, database)
        except NoSuchTableError:
            return []

    def get_sample_data(self, database, table, column=None, operation=None):
        if operation == "hello":
            statement = "SELECT 'Hello World!'"
        elif operation == "model":
            return [], []
        elif operation is not None and operation != "default":
            statement = (
                "SELECT * FROM (%s) LIMIT 1000" % operation
                if operation.strip().lower().startswith("select")
                else operation
            )
        else:
            column = (
                "%(backticks)s%(column)s%(backticks)s"
                % {"backticks": self.backticks, "column": column}
                if column
                else "*"
            )
            statement = textwrap.dedent(
                """\
        SELECT %(column)s
        FROM %(backticks)s%(database)s%(backticks)s.%(backticks)s%(table)s%(backticks)s
        LIMIT %(limit)s
        """
                % {
                    "database": database,
                    "table": table,
                    "column": column,
                    "limit": 100,
                    "backticks": self.backticks,
                }
            )

        connection = self.api._create_connection(self.engine)
        try:
            result = connection.execute(statement)
            return result.cursor.description, result.fetchall()
        finally:
            connection.close()

    def get_keys(self, database, table):
        meta = MetaData()
        metaTable = Table(
            table, meta, schema=database, autoload=True, autoload_with=self.engine
        )

        return {
            "foreign_keys": [
                {"name": fk.parent.name, "to": fk.target_fullname}
                for fk in metaTable.foreign_keys
            ],
            "primary_keys": [{"name": pk.name} for pk in metaTable.primary_key.columns],
        }



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
WSGI config for project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/3.1/howto/deployment/wsgi/
"""

import os

from django.core.wsgi import get_wsgi_application
from whitenoise import WhiteNoise

from compose.conf.settings import STATIC_ROOT

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "compose.conf.settings")


application = get_wsgi_application()

application = WhiteNoise(
    application,
    root=STATIC_ROOT,
)



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path

from . import api

app_name = "connector"

urlpatterns = [
    re_path(r"^types/?$", api.get_types, name="types"),
    re_path(
        r"^instances/?$",
        api.get_instances,
        name="instances",
    ),
    # re_path(r'^api/instance/new/(?P<dialect>[\w\-]+)/(?P<interface>[\w\-]+)$', api.new_connector, name='connectors.api.new_connector'),
    # re_path(r'^api/instance/get/(?P<id>\d+)$', api.get_connector, name='connectors.api.get_connector'),
    # re_path(r'^api/instance/delete/?$', api.delete_connector, name='connectors.api.delete_connector'),
    # re_path(r'^api/instance/update/?$', api.update_connector, name='connectors.api.update_connector'),
    # re_path(r'^api/instance/test/?$', api.test_connector, name='connectors.api.test_connector'),
]



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.encoding import smart_text
from django.utils.translation import gettext as _


class QueryError(Exception):
    def __init__(self, message=None, ex=None):
        self.message = (
            message or str(ex)
            if ex is not None
            else _("No error message, please check the logs.")
        )
        self.ex = ex
        self.extra = {}

    def __unicode__(self):
        return smart_text(self.message)


class SessionExpired(Exception):
    pass


class QueryExpired(Exception):
    def __init__(self, message=None):
        super(QueryExpired, self).__init__()
        self.message = message


class AuthenticationRequired(Exception):
    def __init__(self, message=None):
        super(AuthenticationRequired, self).__init__()
        self.message = message

    def __str__(self):
        return "AuthenticationRequired: %s" % self.message


class OperationTimeout(Exception):
    def __str__(self):
        return "OperationTimeout"


class OperationNotSupported(Exception):
    pass



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.conf.urls.static import static
from django.urls import include, path, re_path
from django.views.generic.base import RedirectView
from drf_spectacular.views import (
    SpectacularAPIView,
    SpectacularRedocView,
    SpectacularSwaggerView,
)
from rest_framework_simplejwt.views import (
    TokenObtainPairView,
    TokenRefreshView,
    TokenVerifyView,
)

import compose.connectors.api as connector_api

urlpatterns = [
    path(
        "",
        RedirectView.as_view(url="/static/", permanent=True),
        name="index",
    ),
]

urlpatterns += [
    re_path("^api/token/auth/?$", TokenObtainPairView.as_view(), name="token_obtain"),
    re_path("^api/token/verify/?$", TokenVerifyView.as_view(), name="token_verify"),
    re_path("^api/token/refresh/?$", TokenRefreshView.as_view(), name="token_refresh"),
    path("api/auth/", include("rest_framework.urls", namespace="rest_framework")),
]

urlpatterns += [
    re_path(r"^api/get_config/?$", connector_api.get_config, name="get_config"),
    re_path(  # To not support anymore going forward, unused
        r"^api/get_namespaces/(?P<interface>[\w\-]+)/?$",
        connector_api.get_namespaces,
        name="get_namespaces",
    ),
]

urlpatterns += [
    path("api/editor/", include("compose.editor.urls", namespace="editor")),
]

urlpatterns += [
    path("connectors/v1/", include("compose.connectors.urls", namespace="connectors"))
]

urlpatterns += [
    path("desktop/", include("compose.connectors.urls_hue4", namespace="hue4-desktop")),
    path("notebook/api/", include("compose.editor.urls_hue4", namespace="hue4-editor")),
]


urlpatterns += [
    path(
        "api/schema/",
        SpectacularAPIView.as_view(),
        name="schema",
    ),
    path(
        "api/schema/swagger-ui/",
        SpectacularSwaggerView.as_view(url_name="schema"),
        name="swagger-ui",
    ),
    path(
        "api/schema/redoc/",
        SpectacularRedocView.as_view(url_name="schema"),
        name="redoc",
    ),
]

urlpatterns += static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class PytestTestRunner:
    """
    Runs pytest to discover and run tests.
    ./manage.py test <django args> -- <pytest args>
    """

    def __init__(self, verbosity=1, failfast=False, keepdb=False, **kwargs):
        self.verbosity = verbosity
        self.failfast = failfast
        self.keepdb = keepdb

    def run_tests(self, test_labels):
        """Run pytest and return the exitcode.

        It translates some of Django's test command option to pytest's.
        """
        import pytest

        argv = []
        if self.verbosity == 0:
            argv.append("--quiet")
        if self.verbosity == 2:
            argv.append("--verbose")
        if self.verbosity == 3:
            argv.append("-vv")
        if self.failfast:
            argv.append("--exitfirst")
        if self.keepdb:
            argv.append("--reuse-db")

        argv.extend(test_labels)
        return pytest.main(argv)



#!/usr/bin/env python
# -- coding: utf-8 --
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Django's command-line utility for administrative tasks."""
import os
import sys
from pathlib import Path


def main():
    """Run administrative tasks."""
    # As we run manage.py from one directory up
    sys.path.append(str(Path(__file__).parent.parent))

    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "compose.conf.settings")
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    execute_from_command_line(sys.argv)


if __name__ == "__main__":
    main()
